# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see guidata/LICENSE for details)

"""
HDF5 I/O demo

DataSet objects may be saved in HDF5 files, a universal hierarchical dataset
file type. This script shows how to save in and then reload data from a HDF5
file.
"""

# guitest: show

import os

from guidata.env import execenv
from guidata.io import HDF5Reader, HDF5Writer
from guidata.qthelpers import qt_app_context
from guidata.tests.dataset.test_all_items import Parameters


def test_loadsave_hdf5():
    """Test HDF5 I/O"""
    fname = "test.h5"
    with qt_app_context():
        if os.path.exists(fname):
            os.unlink(fname)

        e = Parameters()
        if execenv.unattended or e.edit():
            writer = HDF5Writer(fname)
            e.serialize(writer)
            writer.close()

            e = Parameters()
            reader = HDF5Reader(fname)
            e.deserialize(reader)
            reader.close()
            e.edit()
            os.unlink(fname)
        execenv.print("OK")


if __name__ == "__main__":
    test_loadsave_hdf5()



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see guidata/LICENSE for details)

"""Unit tests"""


import unittest

import guidata.dataset as gds
from guidata.dataset.conv import update_dataset
from guidata.env import execenv


class Parameters(gds.DataSet):
    """Example dataset"""

    float1 = gds.FloatItem("float #1", min=1, max=250, help="height in cm")
    float2 = gds.FloatItem("float #2", min=1, max=250, help="width in cm")
    number = gds.IntItem("number", min=3, max=20)


class TestCheck(unittest.TestCase):
    def test_range(self):
        """Test range checking of FloatItem"""
        e = Parameters()
        e.float1 = 150.0
        e.float2 = 400.0
        e.number = 4
        errors = e.check()
        self.assertEqual(errors, ["float2"])

    def test_typechecking(self):
        """Test type checking of FloatItem"""
        e = Parameters()
        e.float1 = 150
        e.float2 = 400
        e.number = 4.0
        errors = e.check()
        self.assertEqual(errors, ["float1", "float2", "number"])

    def test_update(self):
        e1 = Parameters()
        e2 = Parameters()
        e1.float1 = 23
        update_dataset(e2, e1)
        self.assertEqual(e2.float1, 23)


if __name__ == "__main__":
    unittest.main()
    execenv.print("OK")



import warnings

# Compatibility imports with guidata <= 3.3
from guidata.io.base import BaseIOHandler, GroupContext, WriterMixin  # noqa
from guidata.io.h5fmt import HDF5Handler, HDF5Reader, HDF5Writer  # noqa
from guidata.io.inifmt import INIHandler, INIReader, INIWriter  # noqa
from guidata.io.jsonfmt import JSONHandler, JSONReader, JSONWriter  # noqa

warnings.warn(
    "guidata.dataset.io module is deprecated and will be removed in a future release. "
    "Please use guidata.io instead.",
    DeprecationWarning,
    stacklevel=2,
)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2023, Codra, Pierre Raybaut.
# Licensed under the terms of the BSD 3-Clause

"""
genreqs
=======

Module for generating requirements tables in documentation.

This module is derived from the `genreqs.py` module of the `DataLab` project.
"""

from __future__ import annotations

import configparser as cp
import os
import os.path as osp
import re
from types import ModuleType

import requests
import tomli


def extract_requirements_from_toml(path: str) -> dict[str, list[str]] | None:
    """Extract requirements from pyproject.toml file.

    Args:
        path (str): Path to folder containing pyproject.toml file

    Returns:
        dict[str, list[str]]: Dictionary of requirements (main project and optional
         dependencies) or None, if pyproject.toml file does not exist
    """
    filepath = osp.join(path, "pyproject.toml")
    if not osp.isfile(filepath):
        return None
    with open(filepath, "rb") as f:
        data = tomli.load(f)

    requirements = {}

    # Get main project dependencies
    prj = data["project"]
    requirements["main"] = ["Python" + prj["requires-python"]] + prj["dependencies"]

    # Get optional dependencies
    optional_deps = prj.get("optional-dependencies", {})

    requirements.update(optional_deps)
    return requirements


def get_value_from_cfg(config: cp.ConfigParser, section: str, option: str) -> str:
    """Get value from setup.cfg's ConfigParser object.

    Args:
        config (cp.ConfigParser): ConfigParser object
        section (str): Section name
        option (str): Option name

    Returns:
        str: Value
    """
    return config[section][option].strip().splitlines(False)


def extract_requirements_from_cfg(path: str) -> dict[str, list[str]] | None:
    """Extract requirements from setup.cfg file.

    Args:
        filepath (str): Path to folder containing setup.cfg file

    Returns:
        dict[str, list[str]]: Dictionary of requirements (main project and optional
         dependencies) or None, if setup.cfg file does not exist
    """
    filepath = osp.join(path, "setup.cfg")
    if not osp.isfile(filepath):
        return None
    config = cp.ConfigParser()
    config.read(filepath)
    requirements = {
        "main": ["Python" + get_value_from_cfg(config, "options", "python_requires")[0]]
        + get_value_from_cfg(config, "options", "install_requires"),
    }
    for option in config.options("options.extras_require"):
        requirements[option] = get_value_from_cfg(
            config, "options.extras_require", option
        )
    return requirements


def get_package_summary_from_pypi(package: str) -> str:
    """Get package summary from PyPI.

    Args:
        package (str): Package name

    Returns:
        str: Package summary or empty string if package not found on PyPI
    """
    if package == "Python":
        return "Python programming language"
    try:
        response = requests.get(f"https://pypi.org/pypi/{package}/json")
    except requests.exceptions.ConnectionError:
        return ""
    if response.status_code != 200:
        return ""
    return response.json()["info"]["summary"]


def reqlist_to_table(reqs: list[str]) -> str:
    """Convert requirements list to RST table.

    Args:
        reqs (list[str]): Requirements list

    Returns:
        str: RST table
    """
    requirements = [
        ".. list-table::",
        "    :header-rows: 1",
        "    :align: left",
        "",
        "    * - Name",
        "      - Version",
        "      - Summary",
    ]
    modlist = []
    for req in reqs:
        try:
            mod = re.split(" ?(>=|<=|=|<|>)", req)[0]
            ver = req[len(mod) :]
        except ValueError:
            mod, ver = req, ""
        if mod.lower() in modlist:
            continue
        modlist.append(mod.lower())
        requirements.append("    * - " + mod)
        requirements.append("      - " + ver)
        summary = get_package_summary_from_pypi(mod)
        requirements.append("      - " + summary)
    return "\n".join(requirements)


def gen_path_req_rst(
    path: str, modname: str, additional_reqs: list[str], destpath: str | None = None
) -> None:
    """Generate install 'requirements.rst' reStructuredText text.
    This reStructuredText text is written in a file which is by default located in
    the `doc` folder of the module.

    Args:
        path (str): Path to folder containing pyproject.toml or setup.cfg file
        modname (str): Module name
        additional_reqs (list[str]): Additional requirements
        destpath (str): Destination path for requirements.rst file (optional).
    """
    requirements = extract_requirements_from_toml(path)
    if requirements is None:
        requirements = extract_requirements_from_cfg(path)
    if requirements is None:
        raise RuntimeError(
            "Could not find pyproject.toml or setup.cfg file in %s" % path
        )
    requirements = extract_requirements_from_toml(path)
    if requirements is None:
        requirements = extract_requirements_from_cfg(path)
    if requirements is None:
        raise RuntimeError(
            "Could not find pyproject.toml or setup.cfg file in %s" % path
        )
    text = f"""The :mod:`{modname}` package requires the following Python modules:

{reqlist_to_table(requirements["main"]+additional_reqs)}"""
    for category, title in (
        ("dev", "development"),
        ("doc", "building the documentation"),
        ("test", "running test suite"),
    ):
        if category in requirements:
            text += f"""

Optional modules for {title}:

{reqlist_to_table(requirements[category])}"""
    if destpath is None:
        destpath = osp.join(path, "doc")
    with open(osp.join(destpath, "requirements.rst"), "w") as fdesc:
        fdesc.write(text)


def gen_module_req_rst(
    module: ModuleType, additional_reqs: list[str], destpath: str | None = None
) -> None:
    """Generate install 'requirements.rst' reStructuredText text.
    This reStructuredText text is written in a file which is by default located in
    the `doc` folder of the module.

    Args:
        module (ModuleType): Module to generate requirements for
        additional_reqs (list[str]): Additional requirements
        destpath (str): Destination path for requirements.rst file (optional).
    """
    path = osp.abspath(osp.join(osp.dirname(module.__file__), os.pardir))
    gen_path_req_rst(path, module.__name__, additional_reqs, destpath)



from inspect import Parameter, Signature
from typing import Any

from sphinx.application import Sphinx
from sphinx.ext.autodoc import MethodDocumenter, stringify_signature
from sphinx.util.docstrings import prepare_docstring

import guidata.dataset as gds


class CreateMethodDocumenter(MethodDocumenter):
    objtype = "_create_method"
    directivetype = MethodDocumenter.objtype
    priority = 10 + MethodDocumenter.priority
    option_spec = dict(MethodDocumenter.option_spec)
    parent: type[gds.DataSet]

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        try:
            return issubclass(parent, gds.DataSet) and membername == "create"
        except TypeError:
            return False

    def format_signature(self, **kwargs: Any) -> str:
        instance = self.parent()
        params = [
            Parameter(
                item.get_name(),
                Parameter.POSITIONAL_OR_KEYWORD,
                annotation=item.type,
                default=item._default,
            )
            for item in instance.get_items()
        ]
        sig = Signature(parameters=params, return_annotation=self.parent)
        return stringify_signature(sig, **kwargs)

    def get_doc(self):
        self.object.__annotations__["return"] = self.parent
        docstring_lines = [
            f"Returns a new instance of {self.parent.__name__} with the fields set to "
            "the given values.",
            "",
            "Args:",
        ]

        instance = self.parent()
        for item in instance.get_items():
            type_ = item.type
            if not type_:
                type_ = Any

            label = item.get_prop("display", "label")
            if len(label) > 0 and not label.endswith("."):
                label += "."

            help_ = item.get_help(instance)
            if len(help_) > 0 and not help_.endswith("."):
                help_ += "."

            docstring_lines.append(
                f"\t{item.get_name()} ({type_.__name__}): {label} {help_} "
                f"Default: {item._default}"
            )

        docstring_lines.extend(
            ("Returns:", "", f"\tNew instance of {self.parent.__name__}")
        )
        return [
            prepare_docstring(
                "\n".join(docstring_lines),
                tabsize=self.directive.state.document.settings.tab_width,
            )
        ]


def setup(app: Sphinx):
    app.setup_extension("sphinx.ext.autodoc")
    app.add_autodocumenter(CreateMethodDocumenter)



# content of conftest.py

import qtpy

import guidata
from guidata.env import execenv

# Turn on unattended mode for executing tests without user interaction
execenv.unattended = True
execenv.verbose = "quiet"


def pytest_report_header(config):
    """Add additional information to the pytest report header."""
    qtbindings_version = qtpy.PYSIDE_VERSION
    if qtbindings_version is None:
        qtbindings_version = qtpy.PYQT_VERSION
    return [
        f"guidata {guidata.__version__}, "
        f"{qtpy.API_NAME} {qtbindings_version} [Qt version: {qtpy.QT_VERSION}]",
    ]



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see guidata/LICENSE for details)

"""
All guidata item/group features demo
"""

# guitest: show

import atexit
import shutil
import tempfile

import numpy as np

import guidata.dataset as gds
from guidata.dataset.qtitemwidgets import DataSetWidget
from guidata.dataset.qtwidgets import DataSetEditLayout, DataSetShowLayout
from guidata.env import execenv
from guidata.qthelpers import qt_app_context

# Creating temporary files and registering cleanup functions
TEMPDIR = tempfile.mkdtemp(prefix="test_")
atexit.register(shutil.rmtree, TEMPDIR)
FILE_ETA = tempfile.NamedTemporaryFile(suffix=".eta", dir=TEMPDIR)
atexit.register(FILE_ETA.close)
FILE_CSV = tempfile.NamedTemporaryFile(suffix=".csv", dir=TEMPDIR)
atexit.register(FILE_CSV.close)


class SubDataSet(gds.DataSet):
    dir = gds.DirectoryItem("Directory", TEMPDIR)
    fname = gds.FileOpenItem("Single file (open)", ("csv", "eta"), FILE_CSV.name)
    fnames = gds.FilesOpenItem("Multiple files", "csv", FILE_CSV.name)
    fname_s = gds.FileSaveItem("Single file (save)", "eta", FILE_ETA.name)


class SubDataSetWidget(DataSetWidget):
    klass = SubDataSet


class SubDataSetItem(gds.ObjectItem):
    klass = SubDataSet


DataSetEditLayout.register(SubDataSetItem, SubDataSetWidget)
DataSetShowLayout.register(SubDataSetItem, SubDataSetWidget)


class Parameters(gds.DataSet):
    """
    DataSet test
    The following text is the DataSet 'comment': <br>Plain text or
    <b>rich text<sup>2</sup></b> are both supported,
    as well as special characters (α, β, γ, δ, ...)
    """

    dict_ = gds.DictItem(
        "dict_item",
        {
            "strings_col": ["a", "b", "c"],
            "_col": [1, 2.0, 3],
            "float_col": 1.0,
        },
    )
    string = gds.StringItem("String")
    string_regexp = gds.StringItem("String", regexp=r"^[a-z]+[0-9]$", default="abcd9")
    password = gds.StringItem("Password", password=True)
    text = gds.TextItem("Text")
    _bg = gds.BeginGroup("A sub group")
    float_slider = gds.FloatItem(
        "Float (with slider)", default=0.5, min=0, max=1, step=0.01, slider=True
    )
    fl1 = gds.FloatItem(
        "Current", default=10.0, min=1, max=30, unit="mA", help="Threshold current"
    )
    fl2 = gds.FloatItem("Float (col=1)", default=1.0, min=1, max=1).set_pos(col=1)
    fl3 = gds.FloatItem("Not checked float").set_prop("data", check_value=False)
    bool1 = gds.BoolItem("Boolean option without label")
    bool2 = gds.BoolItem("Boolean option with label", "Label").set_pos(col=1, colspan=2)
    color = gds.ColorItem("Color", default="red")
    choice1 = gds.ChoiceItem(
        "Single choice (radio)",
        [(16, "first choice"), (32, "second choice"), (64, "third choice")],
        radio=True,
    ).set_pos(col=1, colspan=2)
    choice2 = gds.ChoiceItem(
        "Single choice (combo)",
        [(16, "first choice"), (32, "second choice"), (64, "third choice")],
    ).set_pos(col=1, colspan=2)
    _eg = gds.EndGroup("A sub group")
    floatarray = gds.FloatArrayItem(
        "Float array", default=np.ones((50, 5), float), format=" %.2e "
    ).set_pos(col=1)
    g0 = gds.BeginTabGroup("group")
    mchoice1 = gds.MultipleChoiceItem(
        "MC type 1", ["first choice", "second choice", "third choice"]
    ).vertical(2)
    mchoice2 = (
        gds.ImageChoiceItem(
            "MC type 2",
            [
                ("rect", "first choice", "gif.png"),
                ("ell", "second choice", "txt.png"),
                ("qcq", "third choice", "html.png"),
            ],
        )
        .set_pos(col=1)
        .set_prop("display", icon="file.png")
        .set_prop("display", size=(32, 32))
    )
    mchoice3 = gds.MultipleChoiceItem(
        "MC type 3", [str(i) for i in range(10)]
    ).horizontal(2)
    eg0 = gds.EndTabGroup("group")
    integer_slider = gds.IntItem(
        "Integer (with slider)", default=5, min=-50, max=50, slider=True
    )
    integer = gds.IntItem("Integer", default=5, min=3, max=6).set_pos(col=1)


def test_all_features():
    """Test all guidata item/group features"""
    with execenv.context(accept_dialogs=True):
        with qt_app_context():
            prm1 = Parameters()
            prm1.floatarray[:, 0] = np.linspace(-5, 5, 50)
            execenv.print(prm1)

            if prm1.edit():
                prm1.edit()
                execenv.print(prm1)
            prm1.view()

            prm2 = Parameters.create(integer=10101010, string="Using `create`")
            assert prm2.integer == 10101010
            print(prm2)

            try:
                # Try to set an unknown attribute using the `create` method:
                Parameters.create(unknown_attribute=42)
            except AttributeError:
                pass
            else:
                raise AssertionError("AttributeError not raised")

            execenv.print("OK")


if __name__ == "__main__":
    test_all_features()



# -*- coding: utf-8 -*-
"""
guidata
=======

Based on the Qt library :mod:`guidata` is a Python library generating graphical
user interfaces for easy dataset editing and display. It also provides helpers
and application development tools for Qt.
"""

__version__ = "3.5.3"


# Dear (Debian, RPM, ...) package makers, please feel free to customize the
# following path to module's data (images) and translations:
DATAPATH = LOCALEPATH = ""


import guidata.config  # noqa: E402, F401


def qapplication():
    """
    Return QApplication instance
    Creates it if it doesn't already exist
    """
    from qtpy.QtWidgets import QApplication

    app = QApplication.instance()
    if not app:
        app = QApplication([])
        install_translator(app)
        set_color_mode(app)
    return app


QT_TRANSLATOR = None


def install_translator(qapp):
    """Install Qt translator to the QApplication instance"""
    global QT_TRANSLATOR
    if QT_TRANSLATOR is None:
        from qtpy.QtCore import QLibraryInfo, QLocale, QTranslator

        locale = QLocale.system().name()
        # Qt-specific translator
        qt_translator = QTranslator()
        paths = QLibraryInfo.location(QLibraryInfo.TranslationsPath)
        for prefix in ("qt", "qtbase"):
            if qt_translator.load(prefix + "_" + locale, paths):
                QT_TRANSLATOR = qt_translator  # Keep reference alive
                break
    if QT_TRANSLATOR is not None:
        qapp.installTranslator(QT_TRANSLATOR)


def set_color_mode(app):
    """Set color mode (dark or light), depending on OS setting"""
    from qtpy.QtCore import Qt
    from qtpy.QtGui import QColor, QPalette
    from qtpy.QtWidgets import QStyleFactory

    from guidata import qthelpers

    if qthelpers.is_dark_mode():
        app.setStyle(QStyleFactory.create("Fusion"))
        dark_palette = QPalette()
        dark_color = QColor(50, 50, 50)
        disabled_color = QColor(127, 127, 127)
        dpsc = dark_palette.setColor
        dpsc(QPalette.Window, dark_color)
        dpsc(QPalette.WindowText, Qt.white)
        dpsc(QPalette.Base, QColor(31, 31, 31))
        dpsc(QPalette.AlternateBase, dark_color)
        dpsc(QPalette.ToolTipBase, Qt.white)
        dpsc(QPalette.ToolTipText, Qt.white)
        dpsc(QPalette.Text, Qt.white)
        dpsc(QPalette.Disabled, QPalette.Text, disabled_color)
        dpsc(QPalette.Button, dark_color)
        dpsc(QPalette.ButtonText, Qt.white)
        dpsc(QPalette.Disabled, QPalette.ButtonText, disabled_color)
        dpsc(QPalette.BrightText, Qt.red)
        dpsc(QPalette.Link, QColor(42, 130, 218))
        dpsc(QPalette.Highlight, QColor(42, 130, 218))
        dpsc(QPalette.HighlightedText, Qt.black)
        dpsc(QPalette.Disabled, QPalette.HighlightedText, disabled_color)
        app.setPalette(dark_palette)
        app.setStyleSheet(
            "QToolTip { color: #ffffff; background-color: #2a82da; border: 1px solid white; }"
        )
